local TracerName, TracerData = ...
--DATE : 18/07/2017

-- ----------------------------------------------------------------------------------------------------------
-- ----------------- Initialisation de la sauvegarde --------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
local TracerDBInit = {
	["Version"] = TracerData.version,
	["Reset"] = {
		--Pour les update tooltip des quêtes d'emissaires
		["EQresetBfAAlliance"] = 0, --compte le nombre de reset daily fait à la connexion (Alliance pour BfA)
		["EQresetBfAHorde"] = 0, --compte le nombre de reset daily fait à la connexion (Horde pour BfA)		
		["EQresetLegion"] = 0, --compte le nombre de reset daily fait à la connexion		
		["resetdaily"] = 0,
		["resetweekly"] = 0,
		["resetbiweekly"] = 0,		
		},
	["Toons"] = {},
	["ToonsCount"] = 0,		--Nombre de Toon déjà enregistré
	["Options"] = {
		minimap = true,		--Pour les donnees de la minimapbutton
		widthrow0 = 240,	--Largueur première colonne
		widthtoon = 45,		--Largueur par Toons
		heightline = 14,	--Hauteur lignes
		police = 12,		--Taille de la police
		anchortop = 700,	--Position par rapport au haut de l'écran
		anchorleft = 723,	--Position par rapport à la gauche de l'écran
		anchortoptoonpanel = 700,	--Position par rapport au haut de l'écran du ToonPanel
		anchorlefttoonpanel = 944,	--Position par rapport à la gauche de l'écran du ToonPanel
		displayStart = true,	--Affichage à la connexion
		displayWindow = 1;		--Fenêtre à afficher (1 Custom, 2 BfA, 3 Legion)
		toonpanelnumber = -1;	--Numéro du Toon affiché dans le ToonPanel
		["DisplayLine"] = {},	--Mémorisation de l'affichage ou non des lignes (catégories 1 à 4)
								--{true/false, true/false}	[1] : Affichage de la ligne [2] : afficher/ne pas affficher la ligne avec le bouton réduire
		["DisplayHeader"] = {},	--Mémorisation de l'affichage ou non des catégories
		["Custom"] = {				--Mémorisation pour l'affichage personnalisée
			["DisplayLine"] = {},
			["DisplayHeader"] = {},
			},
		addonreset = 0,			--Flag pour le reset total de l'addon
		Page = 1,
		Reduction = {},		--Mémoire pour la réduction des catégories
		},
	["ListLine"] = {};
	["WarfrontsBfA"] = {
		["Alliance"] = {
			1,	--Stromgarde
			1,	--Sombrivage
			},
		["Horde"] = {
			1,	--Stromgarde
			1,	--Sombrivage
			},
		},
	["WBBfA"] = "??",
	["WBArgus"] = "??",
	["WBLegion"] = "??",
	["WBNaz"] = "??",
	["WBNzoth"] = "??",	
	["WBSL"] = "??",
	["WBDF"] = "??",
	["SpecializationIdInfos"] = {},
	["AssaultNzothMap"] = {0, 0},	--1 Val, 2 uldum, Reset (1 oui / 0 non)
	["QuestsName"] = {},
	["MythicInfo"] = {},
	["WeeklyDungeonQuestsIdsSL"] = {0, "" ,0, ""},
	["TorghastWings"] = {"", ""},
	["AssaultMawReset"] = 0,
	};

function TracerData.Function:UpdateDB()
	--Reset de l'addon à un changement de version
	if TracerDB.Version < 5.11 then
		TracerDB.Options.addonreset = 1;
	end
	--Reset de l'addon si une ancienne version est installée (pour réinit la database)
	if TracerDB.Version > TracerData.version then
		TracerDB.Options.addonreset = 1;
	end

	--Reset Total de la Database
	if TracerDB.Options.addonreset == 1 then
		TracerData.Function:ResetDatabase();
	end

	--Début modification unique	
	if TracerDB.Version < 5.13 then
		for key, value in pairs(TracerDB.Toons) do			
			value.Quests["DF"]["Quests"][74378] = 0;			
		end		
	end
	if TracerDB.Version < 5.14 then
		for key, value in pairs(TracerDB.Toons) do			
			value.Quests["DF"]["Quests"]["KnowledgePoints"] = {0, 0};	--Terminé, nb de quêtes totale	
			for key1, value1 in pairs(TracerData.WeeklyKnowledgeMaster) do
				value["Quests"]["DF"]["Quests"][value1] = 0;
			end
			for key1, value1 in pairs(TracerData.WeeklyKnowledgeProfessionQuests) do
				for index2, value2 in ipairs(value1) do
					value["Quests"]["DF"]["Quests"][value2] = 0;
				end
			end	
			for key1, value1 in pairs(TracerData.WeeklyKnowledgePoints) do
				for index2, value2 in ipairs(value1) do
					value["Quests"]["DF"]["Quests"][value2[1]] = 0;
				end
			end				
		end		
	end	
	--Fin modification unique
	
	TracerDB.Version = TracerData.version;
end

function TracerData.Function:UpdateDisplays()
	for i=1, #TracerData.windowsorder, 1 do
		--Création des flag DisplayHeader
		if TracerDB.Options.DisplayHeader[i] == nil then
			TracerDB.Options.DisplayHeader[i] = false;
		end
		if TracerDB.Options.Custom.DisplayHeader[i] == nil then
			--Paramétrage pour l'affichage custom de base
			if (i == 2) or (i == 3) or (i == 7) or (i == 8) or (i == 9) or (i == 10) then
				TracerDB.Options.Custom.DisplayHeader[i] = false;	
			else
				TracerDB.Options.Custom.DisplayHeader[i] = true;
			end	
		end
		--Création des flag DisplayLine
		if not TracerDB.Options.DisplayLine[i] then
			TracerDB.Options.DisplayLine[i] = {};
		end
		if not TracerDB.Options.Custom.DisplayLine[i] then
			TracerDB.Options.Custom.DisplayLine[i] = {};
		end
		--Création des flag pour la réduction des catégories
		if TracerDB.Options.Reduction[i] == nil then
			TracerDB.Options.Reduction[i] = false;
		end
	end	
end

--Reset de la Database
function TracerData.Function:ResetDatabase()	
	TracerDB = table.wipe(TracerDB);	--Efface la database
	TracerDB = TracerDBInit;			--Initialise la database
end

-- ----------------------------------------------------------------------------------------------------------
-- ----------------- Gestion des resets  --------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------

--Détermine la date du prochain daily reset
function TracerData.Function:GetNextDailyResetTime()
	local resettime = GetQuestResetTime()
	if not resettime or resettime <= 0 or resettime > 24*3600+30 then
		return nil
	end
	
	return time() + resettime
end
--Détermine la date du prochain weekly reset
function TracerData.Function:GetNextWeeklyResetTime()
	local nightlyReset = TracerData.Function:GetNextDailyResetTime()
	if not nightlyReset then
		return nil	--en cas de problème si il n'y a rien dans la variable
	end 
  
	local t = date("*t",nightlyReset);
	local dayreset = 4;	--numéro du jour pour le reset (4 pour mercred, 3 pour mardi)
	if (TracerData.region == "KR") or (TracerData.region == "EU") then
		dayreset = 4;	--Reset le mercredi pour les regions EU (EU et KR);
	else
		dayreset = 3;	--Reset le mardi pour les regions US	
	end
	
	while t["wday"] ~= dayreset do
		nightlyReset = nightlyReset + 24 * 3600
		t = date("*t",nightlyReset);
	end
	
	return nightlyReset
end
--Déclenchements des resets
function TracerData.Function:Reset()
	--Pour les Weekly
	if time() > TracerDB["Reset"]["resetweekly"] then
		TracerData.Function:ResetWeekly();
	end
	
	TracerDB["Reset"]["resetbiweekly"] = TracerData.Function:GetNextWeeklyResetTime() - 3.5*24*3600;
	--Pour les Bi-Weekly
	if (TracerDB.Reset["flagresetbiweekly"] == 0) and (time() > TracerDB["Reset"]["resetbiweekly"]) then
		TracerData.Function:ResetBiWeekly();
		TracerDB.Reset["flagresetbiweekly"] = 1;
	end	
	
	--Pour les Daily
	local SaveNextDailyReset = TracerDB["Reset"]["resetdaily"];
	local i = 1;
	while TracerData.Function:GetNextDailyResetTime() > SaveNextDailyReset do
		TracerData.Function:ResetDaily();
		SaveNextDailyReset = SaveNextDailyReset + 24*3600;
		if i == 3 then
			break;
		end
		i = i + 1 ;	
	end
	TracerDB["Reset"]["resetdaily"] = TracerData.Function:GetNextDailyResetTime();
end

--Reset des daily
function TracerData.Function:ResetDaily()
	print("Tracer : "..TracerData.L["|cff00ff00Les Daily ont été réinitialisées|r"]);

	--Actualisation des quêtes d'émissaires de Legion et de BfA
	local list = {"EQLegion", "EQBfA"};
	for i = 1, #list, 1 do
		local name = list[i];
		for key, value in pairs(TracerDB.Toons) do
			if TracerDB.Toons[key]["Quests"][name][1] ~= -1 then
				--Quête d'emissaire supplémentaire seulement si la première quête a été faite
				if TracerDB.Toons[key]["Quests"][name]["EQ1"][2] == 0 then
					TracerDB.Toons[key]["Quests"][name][1] = TracerDB.Toons[key]["Quests"][name][1] + 1 ;
				end
				if TracerDB.Toons[key]["Quests"][name][1] > 3 then
					TracerDB.Toons[key]["Quests"][name][1] = 3;
				end		
				--Alternance des quêtes
				for j=1, 2, 1 do
					TracerDB.Toons[key]["Quests"][name]["EQ"..j][1] = TracerDB.Toons[key]["Quests"][name]["EQ"..j+1][1];
					TracerDB.Toons[key]["Quests"][name]["EQ"..j][2] = TracerDB.Toons[key]["Quests"][name]["EQ"..j+1][2];
					TracerDB.Toons[key]["Quests"][name]["EQ"..j][3] = TracerDB.Toons[key]["Quests"][name]["EQ"..j+1][3];				
					TracerDB.Toons[key]["Quests"][name]["EQ"..j][4] = TracerDB.Toons[key]["Quests"][name]["EQ"..j+1][4];				
					TracerDB.Toons[key]["Quests"][name]["EQ"..j][5] = TracerDB.Toons[key]["Quests"][name]["EQ"..j+1][5];				
					TracerDB.Toons[key]["Quests"][name]["EQ"..j][6] = TracerDB.Toons[key]["Quests"][name]["EQ"..j+1][6];				
				end
				TracerDB.Toons[key]["Quests"][name]["EQ3"][1] = 0;
				TracerDB.Toons[key]["Quests"][name]["EQ3"][2] = -1;
				TracerDB.Toons[key]["Quests"][name]["EQ3"][3] = false;				
				TracerDB.Toons[key]["Quests"][name]["EQ3"][4] = " " ;				
				TracerDB.Toons[key]["Quests"][name]["EQ3"][5] = 0;				
				TracerDB.Toons[key]["Quests"][name]["EQ3"][6] = 0;								
			end
		end
	end
	
	if TracerDB["Reset"]["EQresetBfAAlliance"] < 3 then
		TracerDB["Reset"]["EQresetBfAAlliance"] = TracerDB["Reset"]["EQresetBfAAlliance"]+1;
	end
	if TracerDB["Reset"]["EQresetBfAHorde"] < 3 then
		TracerDB["Reset"]["EQresetBfAHorde"] = TracerDB["Reset"]["EQresetBfAHorde"]+1;
	end	
	if TracerDB["Reset"]["EQresetLegion"] < 3 then
		TracerDB["Reset"]["EQresetLegion"] = TracerDB["Reset"]["EQresetLegion"]+1;
	end	

	for key, value in pairs(TracerDB.Toons) do
		--Reset du 1er donjon héro
		if (value["Quests"]["HD"] ~= -1) and (value["Ilvl"] >= 155 )then
			value["Quests"]["HD"] = 0; --Flag du donjon héro à 0
		end
		--Reset de la Foire de Sombrelune		
		for index1, value1 in ipairs(value["Quests"]["Darkmoon"]["Daily"]) do
			value["Quests"]["Darkmoon"]["Daily"][index1] = 0;
		end	
		--Reset des quêtes de Nazjatar
		if	value.Level >= 50 then
			value["Quests"]["BfA"]["Nazjatar"]["Daily"] = 0;
		end	
		--Reset des quêtes de Mechagon		
		if	value.Level >= 50 then
			value["Quests"]["BfA"]["Mechagon"] = {-1, -1, -1};
		end
		--Reset Mini-Vision de N'Zoth		
		value["Quests"]["BfA"]["MiniVision"] = 0;		
		--Reset des Callings (SL)
		value.Quests.SL.Callings[1] = value.Quests.SL.Callings[2];
		value.Quests.SL.Callings[2] = value.Quests.SL.Callings[3];
		value.Quests.SL.Callings[3] = {0, false};
		--Reset PVP
		value["PVP"]["Daily"] = {0, 0, 0};
		value["PVP"]["DailyRated"] = {0, 0, 0};
		--Reset Korthia Daily
		value.Quests.SL.KorthiaDaily = 0;
		--Reset "De l'amour dans l'air"
		value.LoveIsInTheAir = {0, 0};
		--Reset daily event prépatch DF
		value.Quests.DF.Quests[70049] = -1
	end
end

--Reset des weekly
function TracerData.Function:ResetWeekly()
	print("Tracer : "..TracerData.L["|cffff0000Les Weekly ont été réinitialisées|r"]);
	TracerDB["Reset"]["resetweekly"] = TracerData.Function:GetNextWeeklyResetTime();
	
	--Mise à jour du coffe m+ (à faire avant le reset du donjon m+ pour ne pas écraser les données)
	for key, value in pairs(TracerDB.Toons) do
		if value["Quests"]["MD"] >= 1 then
			value["Quests"]["MC"] = 0;
			TracerDB.Options.DisplayLine[1][5][1] = true;	--pour faire apparaitre la ligne				
		elseif value["Level"] == 60 then
			value["Quests"]["MC"] = 1;
		else
			value["Quests"]["MC"] = -1;
		end
	end
	
	--Reset du donjon mythique de la semaine et vérouillage de mythic
	for key, value in pairs(TracerDB.Toons) do
		if value["Quests"]["MD"] ~= -2 then
			value["Quests"]["MD"] = -1; --Flag donjon m+		
		end
		for key1, value1 in pairs(value.Mythic) do
			value.Mythic[key1][1] = false;			
			value.Mythic[key1][2] = -1;
			value.Mythic[key1][3] = -1;
		end		
	end	

	--Reset des clés mythique de la semaine
	for key, value in pairs(TracerDB.Toons) do
		if value.Level == 70 then
			value["Quests"]["MK"] = {0, 0, 0, 0, 0, 0};
		end
	end

	for key, value in pairs(TracerDB.Toons) do	
		--Reset du WB de BfA	
		if value["Level"] >= 50 then
			value["Quests"]["WBBfA"] = 0; --Flag à 0
			value["Quests"]["WBNzoth"] = 0; --Flag à 0			
		end
		--Reset du WB de Legion		
		if value["Level"] >= 50 then
			value["Quests"]["WBLegion"] = 0; --Flag à 0	
		end	
		--Reset du WB de Nazjatar
		if value["Level"] >= 50 then
			value["Quests"]["WBNaz"] = 0; --Flag à 0	
		end
		--Reset du WB de SL	
		if value["Level"] >= 60 then
			value["Quests"]["SL"]["WBSL"] = 0; --Flag à 0	
		end	
		--Reset du WB d'Argus
		if value["Quests"]["WBArgus"] ~= -1 then
			value["Quests"]["WBArgus"] = 0; --Flag à 0
		end	
		--Reset WB Korthia
		if value.Quests.SL.KorthiaWB ~= -1 then
			value.Quests.SL.KorthiaWB = 0;
		end
		--Reset WB ZM
		if value.Quests.SL.ZMWB ~= -1 then
			value.Quests.SL.ZMWB = 0;
		end	
		--Reset WB
		if value["Quests"]["DF"]["WBDF"] ~= -1 then
			value["Quests"]["DF"]["WBDF"] = 0;
		end			

		--Reset des quêtes d'Argus
		for index1, value1 in ipairs(value["Quests"]["Argus"]) do
			if (index1 ~= 5) and (value1 ~= -1) then 
				value["Quests"]["Argus"][index1] = 0;
			end			
		end
		--Reset des weekly de Nazjatar
		if value["Level"] >= 50 then
			value["Quests"]["BfA"]["Nazjatar"]["Weekly"] = {0, 0, 0};
		end	
		--Reset des Weekly Dungeons de SL
		value.Quests.SL.WeeklyDungeonQuests = {-1, -1};		
		--Reset weekly SL
		value.Quests.SL.Quests[1] = 0;				--Âmes perdues
		if value.Quests.SL.Quests[2] == 1001 then
			value.Quests.SL.Quests[2] = -1;			--1k anima
		end
		--Reset de l'assaut de congrégation, weekly kothia
		value.Quests.SL.AssaultMaw = 0;
		if value.Quests.SL.KorthiaWeekly == 101 then
			value.Quests.SL.KorthiaWeekly = -1;
		end
		--Reset Korthia WQ
		if value.Quests.SL.KorthiaWQ ~= -1 then
			value.Quests.SL.KorthiaWQ = 0;
		end	
		--Reset ZM Weekly
		if value.Quests.SL.ZMWeekly == 101 then
			value.Quests.SL.ZMWeekly = -1;
		end
		--Reset WeeklyChests de DF
		value.Quests["DF"]["Quests"][71995] = 0;
		value.Quests["DF"]["Quests"][70906] = 0;
		value.Quests["DF"]["Quests"][70893] = 0;
		value.Quests["DF"]["Quests"][71033] = 0;
		value.Quests["DF"]["Quests"][70866] = 0;
		value.Quests["DF"]["Quests"]["WeeklyChests"] = 0;
		--Reset de la weekly de Valdrakken
		value.Quests["DF"]["Quests"]["WeeklyValdrakken"] = {-1, 0};
		--Reset Irion et Sabellian
		if (value.Quests["DF"]["Quests"][66133] == 1) or (value.Quests["DF"]["Quests"][66805] == 1) then
			value.Quests["DF"]["Quests"][66133] = 0;		
			value.Quests["DF"]["Quests"][66805] = 0;		
		end
		--Reset des expéditions d'îles
		if value["Quests"]["BfA"]["IE"][1] == 1 then
			value["Quests"]["BfA"]["IE"][1] = 0;
			value["Quests"]["BfA"]["IE"][2] = 0;
		end	
		--Reset des sceaux
		value.Quests.BfA.Seal = 0;			
		--Reset des points de connaissance de métiers
		value["Quests"]["KnowledgePoints"] = {0, 0};	
		--Reset des quêtes de connaissance de métiers
		for key, value in pairs(TracerData.WeeklyKnowledgeMaster) do
			TracerDB["Toons"][TracerData.thisToon]["Quests"]["DF"]["Quests"][value] = 0;
		end
		for key, value in pairs(TracerData.WeeklyKnowledgeProfessionQuests) do
			for index1, value1 in ipairs(value) do
				TracerDB["Toons"][TracerData.thisToon]["Quests"]["DF"]["Quests"][value1] = 0;
			end
		end	
		for key, value in pairs(TracerData.WeeklyKnowledgePoints) do
			for index1, value1 in ipairs(value) do
				TracerDB["Toons"][TracerData.thisToon]["Quests"]["DF"]["Quests"][value1[1]] = 0;
			end
		end
		
		--Reset vérouillages d'instances		
		value.Instances = {};	
		--Reset List des mm+
		value.MythicList = {};		
	end
	TracerDB.WBLegion = "??";	
	TracerDB.WBBfA = "??";
	TracerDB.WBNzoth = "??";	
	TracerDB.WBNaz = "??";
	TracerDB.WBSL = "??";
	TracerDB.WBArgus = "??";
	TracerDB.WBDF = "??";
	TracerDB["WeeklyDungeonQuestsIdsSL"] = {0, "", 0, ""};	
	TracerDB.AssaultMawReset = 0;	
	
	--Reset de l'évènement hebdo
	for key, value in pairs(TracerDB.Toons) do
		--Timewalking dungeons
		value.Quests["TD"] = {
			-1, --Compteur Timewalking de la semaine (-1 quête non prise)
			false,	--Rendu ou non
			-1,	--Quête du premier de la semaine
			-1, --Quête du raid
			};
		--weeklyquest
		value.Quests["Weeklyquest"] = {
			-1,	--flag quête complétée
			-1,	--questDone
			-1,	--questNeed
			};	
	end
	--Reset de la Foire de Sombrelune
	local Date = date("*t");
	if Date.day >= 14 then
		for key, value in pairs(TracerDB.Toons) do
			for key1, value1 in pairs(value["Quests"]["Darkmoon"]) do
				for index2, value2 in ipairs(value1) do
					value2 = -1;
				end
			end	
		end	
	end	
	--Reset Assaults N'zoth
	for key, value in pairs(TracerDB.Toons) do
		if value["Level"] >= 50 then
			value["Quests"]["BfA"]["Assaults"] = {0, 0};
			value["Quests"]["BfA"]["MiniVision"] = 0;
		end
	end
	if TracerDB.AssaultNzothMap[1] == 1 then
		TracerDB.AssaultNzothMap = {2, 0};
	else
		TracerDB.AssaultNzothMap = {1, 0};	
	end	
	--Reset du PVP
	for key, value in pairs(TracerDB.Toons) do
		value["PVP"]["CC"] = {0, 0};
		value["PVP"]["CTAQuests"] = {};
		value["PVP"]["BattleNazjatar"] = {0, 0};
		if value["PVP"]["25kill"] ~= -2 then
			value["PVP"]["25kill"] = -1;
		end
	end	
	
	--Reset de Torghast
	-- for key, value in pairs(TracerDB.Toons) do
		-- for key1, value1 in pairs(value.Torghast) do
			-- TracerDB.Toons[key]["Torghast"][key1] = false;
		-- end
	-- end	
	
	--Reset Nom des ailes de Torghast
	-- TracerDB.TorghastWings = {"", ""};
	
	TracerData.Function:ResetBiWeekly();
	TracerDB.Reset["flagresetbiweekly"] = 0;	
end

--Reset des Bi-Weekly
function TracerData.Function:ResetBiWeekly()
	print("Tracer : "..TracerData.L["|cfa913f00Les Bi-Weekly ont été réinitialisées|r"]);

	for key, value in pairs(TracerDB.Toons) do	
		--Reset des WQ Courses Draconique
		value.Quests.DF.Quests.CourseDraconique	= 0; --Flag à 0			
	end
end

-- ----------------------------------------------------------------------------------------------------------
-- ----------------- Lancement de l'addon  ------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------

TracerData.Core = CreateFrame("FRAME");
TracerData.Core:RegisterEvent("ADDON_LOADED");
TracerData.Core:RegisterEvent("PLAYER_ENTERING_WORLD");
TracerData.Core:RegisterEvent("PLAYER_LOGOUT");
TracerData.Core:SetScript("OnEvent", function(_, event, arg1)
	if event == "ADDON_LOADED" and arg1 == "Tracer" then
		TracerData.Function:CoreInit();	
	end
	if event == "PLAYER_ENTERING_WORLD" then
		TracerData.Core:UnregisterEvent("PLAYER_ENTERING_WORLD");		
		TracerData.Function:Core();		
	end	
	if event == "PLAYER_LOGOUT" then

	end	
	end);

function TracerData.Function:CoreInit(event, arg1)
	--Initialise la sauvegarde (première utilisation)
		if TracerDB.Toons == nil then
			TracerDB = TracerDBInit;
		end
	--Update de la DB (changement de version ou reset manuel)
		TracerData.Function:UpdateDB();
	--Création des mémoires pour l'affichage
		TracerData.Function:UpdateDisplays();
	--Localisation
		if (GetLocale() == "frFR") then
			TracerData.L = TracerData.Lfrench;		
		else
			TracerData.L = TracerData.Lenglish;
		end
	--Region
	local regionID = GetCurrentRegion();
	TracerData.region = ({ "US", "KR", "EU", "TW", "CN" })[regionID];
end

function TracerData.Function:Core()
	--Reset Daily/Weekly
		TracerData.Function:Reset();
	--Ajout/Update du personnage
		TracerData.Function:InitToon();
	--Création de la liste des Toons à afficher
		TracerData.Function:ListToons();
	--Initialisation Monnaies/Reput/Torghast
		TracerData.Function:InitCRT();
	--Création de la frame pour les scan sur timer
		TracerData.TracerScan = CreateFrame("Frame", "TracerScan", UIParent);
	--Affichage
		TracerData.Function:WindowCreateGeneralInfos();
		TracerData.Function:WindowCreateDragonflight();
		TracerData.Function:WindowCreateShadowlands();		
		TracerData.Function:WindowCreateBfA();
		TracerData.Function:WindowCreateLegion();
		TracerData.Function:WindowCreateCurrencies();		
		TracerData.Function:WindowCreateReputations();
		TracerData.Function:WindowCreateRaidLockout();	
		TracerData.Function:WindowCreatePVP();	
		TracerData.Function:ToonPanelCreate();		--ToonPanel sans les talents
		--TracerData.Function:ToonPanelCreate2();	--ToonPanel avec les talents
	--Affiche la fenêtre si DisplayStart	
		if TracerDB.Options.displayStart then
			if TracerDB.Toons[TracerData.thisToon]["Order"] <= 10 then
				TracerDB.Options.Page = 1;			
			elseif (TracerDB.Toons[TracerData.thisToon]["Order"] >= 11) and (TracerDB.Toons[TracerData.thisToon]["Order"] <= 20) then
				TracerDB.Options.Page = 2;			
			elseif (TracerDB.Toons[TracerData.thisToon]["Order"] >= 21) and (TracerDB.Toons[TracerData.thisToon]["Order"] <= 30) then
				TracerDB.Options.Page = 3;
			elseif (TracerDB.Toons[TracerData.thisToon]["Order"] >= 31) and (TracerDB.Toons[TracerData.thisToon]["Order"] <= 40) then
				TracerDB.Options.Page = 4;
			else
				TracerDB.Options.Page = 5;				
			end			
			C_Timer.After(3, function() TracerData.Headers[1]:Show(); end);			
		end
	--Quelle fenêtre afficher
		if TracerDB.Options.displayWindow == 1 then
			TracerData.Function:DisplayCustom();
		elseif TracerDB.Options.displayWindow == 2 then
			TracerData.Function:DisplayBfA();
		elseif TracerDB.Options.displayWindow == 3 then
			TracerData.Function:DisplayLegion();
		elseif TracerDB.Options.displayWindow == 4 then
			TracerData.Function:DisplayWoD();
		elseif TracerDB.Options.displayWindow == 5 then
			TracerData.Function:DisplaySL();
		elseif TracerDB.Options.displayWindow == 6 then
			TracerData.Function:DisplayDF();			
		end		
	--Forcer le menu options à s'ouvrir au démarrage
		--TracerData.Function:OptionsTracer()
	--Début du scan sur timer
		TracerData.TracerScan:SetScript("OnUpdate", function(self, elapsed)
			TracerData.TimeSinceLastUpdate = TracerData.TimeSinceLastUpdate + elapsed
			TracerData.TimeSinceLastUpdate2 = TracerData.TimeSinceLastUpdate2 + elapsed			
			if TracerData.TimeSinceLastUpdate >= TracerData.ONUPDATE_INTERVAL then
				--Warfronts
				if UnitFactionGroup("player") ~= "Neutral" then
					TracerData.Function:WarfrontsBfAChecking();
				end
				--Assault de Faction
				TracerData.Function:AssaultsResetChecking();
				--Mise à jour des Tooltip pour les Emisaires
				TracerData.Function:UpdateEQTooltip("BfA");				
				TracerData.Function:UpdateEQTooltip("Legion");	
				--Reset du timer
				TracerData.TimeSinceLastUpdate = 0;
			end
			if TracerData.verifmm > 0 then
				if TracerData.TimeSinceLastUpdate2 >= TracerData.ONUPDATE_INTERVAL2 then
					--MM+
					C_MythicPlus.RequestMapInfo();
					C_MythicPlus.RequestRewards();	
					local MapTable = C_ChallengeMode.GetMapTable();
					for key, value in pairs(MapTable) do
						C_ChallengeMode.RequestLeaders(value);
					end
					--Reset du timer
					TracerData.TimeSinceLastUpdate2 = 0;
				TracerData.verifmm = TracerData.verifmm - 1;					
				end
			end			
		end)				
	--Début Scan
		TracerData.Function:ScanInit();
	--Affichage du boutton sur la minimap
		if TracerDB.Options.minimap then
			LibStub("LibDBIcon-1.0"):Show(TracerName);
		else
			LibStub("LibDBIcon-1.0"):Hide(TracerName);
		end	
end

function TracerData.Function:ScanInit()
--Gestion Affichage des catégories EH, DF, WE, et métiers du Toons
	C_Timer.After(5, function() TracerData.Function:WindowCreateWeeklyEvent1(); end);	
	C_Timer.After(6, function() TracerData.Function:WindowCreateDarkmoonFaire1(); end);
	C_Timer.After(7, function() TracerData.Function:WindowCreateTimewalking1(); end);	
	
--Gestion Affichage des évènements mondiaux
	C_Timer.After(8, function() TracerData.Function:WindowCreateLoveIsInTheAir(); end);
--PO
	TracerDB["Toons"][TracerData.thisToon]["PO"] = GetMoney() / 10000 - GetMoney() % 10000 / 10000 ;
	
	if not TracerDB.Toons[TracerData.thisToon]["Display"] then return end
--GeneralInfos	
	TracerData.Function:IlvlChecking();
	TracerData.Function:HerodayChecking();
	C_MythicPlus.RequestMapInfo();
	for key, value in ipairs(C_ChallengeMode.GetMapTable()) do
		C_ChallengeMode.RequestLeaders(value);
	end	
	TracerData.Function:MythicPlusChest();	
	TracerData.Function:MythicPlusKey();	
--DF
	TracerData.Function:DragonflightChecking();
--SL	
	C_CovenantCallings.RequestCallings();
	TracerData.Function:ShadowlandsQuestsChecking();
	TracerData.Function:WeeklyDungeonsQuestsChecking();
	TracerData.Function:WorldBossCheckingSL();
	TracerData.Function:AssaultMawResetChecking();
--BfA
	--C_Timer.After(10, function() TracerData.Function:UpdateEQTooltip("BfA"); end)
	TracerData.Function:QuestsCheckingBfA();
	--C_Timer.After(5, function() TracerData.Function:AzeriteChecking(); end);	
	TracerData.Function:WorldBossCheckingBfA();
	if UnitFactionGroup("player") ~= "Neutral" then
		TracerData.Function:WarfrontToonChecking(1, TracerDB.WarfrontsBfA[UnitFactionGroup("player")][1], 5);	
		TracerData.Function:WarfrontToonChecking(2, TracerDB.WarfrontsBfA[UnitFactionGroup("player")][2], 6);
	end
	TracerData.Function:QuestsCheckingBfANazjatar();
	TracerData.Function:VesselsChecking();	
--Legion
	--C_Timer.After(10, function() TracerData.Function:UpdateEQTooltip("Legion"); end)
	TracerData.Function:WorldBossCheckingLegion();
	TracerData.Function:QuestsCheckingArgus();
	TracerData.Function:WorldBossCheckingArgus();	
--PVP
	TracerData.Function:PVPConquestCap();
	TracerData.Function:PVPQuestsUpdate();
	RequestPVPRewards();
--Currencies
	TracerData.Function:UpdateCurrencies();
--Reputations
	TracerData.Function:UpdateFactions();
--RaidLockout

--WeeklyEvent

--DarkmoonFaire

--WorldEvent : L'amour est dans l'air

--Jobs
	TracerData.Function:UpdateJobsToon();
--Specialisation
	TracerData.Function:NodesUpdate();	
--Inventory
	C_Timer.After(5, function() TracerData.Function:InventoryUpdate(); end)	
	C_Timer.After(6, function() TracerData.ToonPanel["Scan"]:RegisterEvent("UNIT_INVENTORY_CHANGED"); end)

--Affichage
	TracerData.Function:WindowUpdateDragonflight();		
	TracerData.Function:WindowUpdateShadowlands();
	TracerData.Function:WindowUpdateBfA();
	
	TracerData.Function:WindowUpdateReputations();
	TracerData.Function:WindowUpdateCurrencies();	
end
